# RCS1077: Optimize LINQ method call

| Property | Value   |
| -------- | ------- |
| Id       | RCS1077 |
| Severity | Info    |

## Examples

### Code with Diagnostic

```csharp
bool any = items.Where(predicate).Any();
```

### Code with Fix

```csharp
bool any = items.Any(predicate);
```

- - -

### Code with Diagnostic

```csharp
int max = items.Select(selector).Max();
```

### Code with Fix

```csharp
int max = items.Max(selector);
```

- - -

### Code with Diagnostic

```csharp
IEnumerable<Foo> x = items.Where(f => f is Foo).Cast<Foo>();
```

### Code with Fix

```csharp
IEnumerable<Foo> x = items.OfType<Foo>();
```

- - -

### Code with Diagnostic

```csharp
bool x = items.Where((f) => Foo1(f)).Any(f => Foo2(f));
```

### Code with Fix

```csharp
bool x = items.Any((f) => Foo1(f) && Foo2(f));
```

- - -

### Code with Diagnostic

```csharp
IEnumerable<object> x = items.Select(f => (object)f);
```

### Code with Fix

```csharp
IEnumerable<object> x = items.Cast<object>();
```

- - -

### Code with Diagnostic

```csharp
bool x = items.FirstOrDefault((f) => Foo(f)) != null;
```

### Code with Fix

```csharp
bool x = items.Any((f) => Foo(f));
```

- - -

### Code with Diagnostic

```csharp
bool x = items.FirstOrDefault() != null;
```

### Code with Fix

```csharp
bool x = items.Any();
```

- - -

### Code with Diagnostic

```csharp
if (enumerable.Count() != 0) // RCS1077
{
}
```

### Code with Fix

```csharp
if (enumerable.Any())
{
}
```

- - -

### Code with Diagnostic

```csharp
if (list.Count() == 1) // RCS1077
{
}
```

### Code with Fix

```csharp
if (list.Count == 1)
{
}
```

- - -

### Code with Diagnostic

```csharp
var stack = new Stack<object>();
// ...
object x = stack.First(); // RCS1077
```

### Code with Fix

```csharp
var stack = new Stack<object>();
// ...
object x = stack.Peek();
```

- - -

### Code with Diagnostic

```csharp
var queue = new Queue<object>();
// ...
object x = queue.First(); // RCS1077
```

### Code with Fix

```csharp
var queue = new Queue<object>();
// ...
object x = queue.Peek();
```

- - -

### Code with Diagnostic

```csharp
enumerable.Any() ? enumerable.First() : default
```

### Code with Fix

```csharp
enumerable.FirstOrDefault()
```

- - -

### Code with Diagnostic

```csharp
enumerable.OrderBy(f => f).Reverse()
```

### Code with Fix

```csharp
enumerable.OrderByDescending()
```

- - -

### Code with Diagnostic

```csharp
enumerable.SelectMany(f => f.Items).Count()
```

### Code with Fix

```csharp
enumerable.Sum(f => f.Items.Count)
```

- - -

### Code with Diagnostic

```csharp
listOfT.Select(f => M(f)).ToList()
```

### Code with Fix

```csharp
listOfT.ConvertAll(f => M(f))
```


*\(Generated with [DotMarkdown](http://github.com/JosefPihrt/DotMarkdown)\)*