# [SyntaxNode](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.syntaxnode) Class Extensions

[Home](../../../README.md)

| Extension Method | Summary |
| ---------------- | ------- |
| [AppendToLeadingTrivia\<TNode\>(TNode, IEnumerable\<SyntaxTrivia\>)](../../../Roslynator/SyntaxExtensions/AppendToLeadingTrivia/README.md#3159857401) | Creates a new node from this node with the leading trivia replaced with a new trivia where the specified trivia is added at the end of the leading trivia\. |
| [AppendToLeadingTrivia\<TNode\>(TNode, SyntaxTrivia)](../../../Roslynator/SyntaxExtensions/AppendToLeadingTrivia/README.md#161505572) | Creates a new node from this node with the leading trivia replaced with a new trivia where the specified trivia is added at the end of the leading trivia\. |
| [AppendToTrailingTrivia\<TNode\>(TNode, IEnumerable\<SyntaxTrivia\>)](../../../Roslynator/SyntaxExtensions/AppendToTrailingTrivia/README.md#782693212) | Creates a new node from this node with the trailing trivia replaced with a new trivia where the specified trivia is added at the end of the trailing trivia\. |
| [AppendToTrailingTrivia\<TNode\>(TNode, SyntaxTrivia)](../../../Roslynator/SyntaxExtensions/AppendToTrailingTrivia/README.md#3044430369) | Creates a new node from this node with the trailing trivia replaced with a new trivia where the specified trivia is added at the end of the trailing trivia\. |
| [ContainsDirectives(SyntaxNode, TextSpan)](../../../Roslynator/SyntaxExtensions/ContainsDirectives/README.md) | Returns true if the node contains any preprocessor directives inside the specified span\. |
| [ContainsUnbalancedIfElseDirectives(SyntaxNode, TextSpan)](../../../Roslynator/CSharp/SyntaxExtensions/ContainsUnbalancedIfElseDirectives/README.md#1293653798) | Returns true if the specified node contains `#if` directive but it does not contain related `#endif` directive\. |
| [ContainsUnbalancedIfElseDirectives(SyntaxNode)](../../../Roslynator/CSharp/SyntaxExtensions/ContainsUnbalancedIfElseDirectives/README.md#3758239446) | Returns true if the specified node contains `#if` directive but it does not contain related `#endif` directive\. |
| [FirstAncestor(SyntaxNode, Func\<SyntaxNode, Boolean\>, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestor/README.md#4122783476) | Gets the first ancestor that matches the predicate\. |
| [FirstAncestor(SyntaxNode, SyntaxKind, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestor/README.md#2319682747) | Gets the first ancestor of the specified kind\. |
| [FirstAncestor(SyntaxNode, SyntaxKind, SyntaxKind, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestor/README.md#3436882574) | Gets the first ancestor of the specified kinds\. |
| [FirstAncestor(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestor/README.md#2233565705) | Gets the first ancestor of the specified kinds\. |
| [FirstAncestor\<TNode\>(SyntaxNode, Func\<TNode, Boolean\>, Boolean)](../../../Roslynator/SyntaxExtensions/FirstAncestor/README.md) | Returns the first node of type **TNode** that matches the predicate\. |
| [FirstAncestorOrSelf(SyntaxNode, Func\<SyntaxNode, Boolean\>, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestorOrSelf/README.md#3332998863) | Gets the first ancestor that matches the predicate\. |
| [FirstAncestorOrSelf(SyntaxNode, SyntaxKind, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestorOrSelf/README.md#706908351) | Gets the first ancestor of the specified kind\. |
| [FirstAncestorOrSelf(SyntaxNode, SyntaxKind, SyntaxKind, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestorOrSelf/README.md#1839797174) | Gets the first ancestor of the specified kinds\. |
| [FirstAncestorOrSelf(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/FirstAncestorOrSelf/README.md#280718491) | Gets the first ancestor of the specified kinds\. |
| [FirstDescendant\<TNode\>(SyntaxNode, Func\<SyntaxNode, Boolean\>, Boolean)](../../../Roslynator/SyntaxExtensions/FirstDescendant/README.md#3727489774) | Searches a list of descendant nodes in prefix document order and returns first descendant of type **TNode**\. |
| [FirstDescendant\<TNode\>(SyntaxNode, TextSpan, Func\<SyntaxNode, Boolean\>, Boolean)](../../../Roslynator/SyntaxExtensions/FirstDescendant/README.md#2271502195) | Searches a list of descendant nodes in prefix document order and returns first descendant of type **TNode**\. |
| [FirstDescendantOrSelf\<TNode\>(SyntaxNode, Func\<SyntaxNode, Boolean\>, Boolean)](../../../Roslynator/SyntaxExtensions/FirstDescendantOrSelf/README.md#4205056015) | Searches a list of descendant nodes \(including this node\) in prefix document order and returns first descendant of type **TNode**\. |
| [FirstDescendantOrSelf\<TNode\>(SyntaxNode, TextSpan, Func\<SyntaxNode, Boolean\>, Boolean)](../../../Roslynator/SyntaxExtensions/FirstDescendantOrSelf/README.md#3421526450) | Searches a list of descendant nodes \(including this node\) in prefix document order and returns first descendant of type **TNode**\. |
| [GetFirstDirective(SyntaxNode, TextSpan, Func\<DirectiveTriviaSyntax, Boolean\>)](../../../Roslynator/CSharp/SyntaxExtensions/GetFirstDirective/README.md) | Gets the first directive of the tree rooted by this node\. |
| [GetLeadingAndTrailingTrivia(SyntaxNode)](../../../Roslynator/SyntaxExtensions/GetLeadingAndTrailingTrivia/README.md) | Returns leading and trailing trivia of the specified node in a single list\. |
| [IsDescendantOf(SyntaxNode, SyntaxKind, Boolean)](../../../Roslynator/CSharp/SyntaxExtensions/IsDescendantOf/README.md) | Returns true if a node is a descendant of a node with the specified kind\. |
| [IsInExpressionTree(SyntaxNode, SemanticModel, CancellationToken)](../../../Roslynator/CSharp/SyntaxExtensions/IsInExpressionTree/README.md) | Determines if the specified node is contained in an expression tree\. |
| [IsKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsKind/README.md#1975403005) | Returns true if a node's kind is one of the specified kinds\. |
| [IsKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsKind/README.md#3028059531) | Returns true if a node's kind is one of the specified kinds\. |
| [IsKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsKind/README.md#580360109) | Returns true if a node's kind is one of the specified kinds\. |
| [IsKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsKind/README.md#3185459241) | Returns true if a node's kind is one of the specified kinds\. |
| [IsKind(SyntaxNode, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsKind/README.md#1541609469) | Returns true if a node's kind is one of the specified kinds\. |
| [IsParentKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsParentKind/README.md#3570688798) | Returns true if a node parent's kind is one of the specified kinds\. |
| [IsParentKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsParentKind/README.md#2131485272) | Returns true if a node parent's kind is one of the specified kinds\. |
| [IsParentKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsParentKind/README.md#1935129073) | Returns true if a node parent's kind is one of the specified kinds\. |
| [IsParentKind(SyntaxNode, SyntaxKind, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsParentKind/README.md#2974797413) | Returns true if a node parent's kind is one of the specified kinds\. |
| [IsParentKind(SyntaxNode, SyntaxKind, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsParentKind/README.md#93375457) | Returns true if a node parent's kind is one of the specified kinds\. |
| [IsParentKind(SyntaxNode, SyntaxKind)](../../../Roslynator/CSharp/SyntaxExtensions/IsParentKind/README.md#1778613464) | Returns true if a node parent's kind is the specified kind\. |
| [PrependToLeadingTrivia\<TNode\>(TNode, IEnumerable\<SyntaxTrivia\>)](../../../Roslynator/SyntaxExtensions/PrependToLeadingTrivia/README.md#3915245611) | Creates a new node from this node with the leading trivia replaced with a new trivia where the specified trivia is inserted at the beginning of the leading trivia\. |
| [PrependToLeadingTrivia\<TNode\>(TNode, SyntaxTrivia)](../../../Roslynator/SyntaxExtensions/PrependToLeadingTrivia/README.md#3276186521) | Creates a new node from this node with the leading trivia replaced with a new trivia where the specified trivia is inserted at the beginning of the leading trivia\. |
| [PrependToTrailingTrivia\<TNode\>(TNode, IEnumerable\<SyntaxTrivia\>)](../../../Roslynator/SyntaxExtensions/PrependToTrailingTrivia/README.md#1111873538) | Creates a new node from this node with the trailing trivia replaced with a new trivia where the specified trivia is inserted at the beginning of the trailing trivia\. |
| [PrependToTrailingTrivia\<TNode\>(TNode, SyntaxTrivia)](../../../Roslynator/SyntaxExtensions/PrependToTrailingTrivia/README.md#3683468027) | Creates a new node from this node with the trailing trivia replaced with a new trivia where the specified trivia is inserted at the beginning of the trailing trivia\. |
| [RemoveTrivia\<TNode\>(TNode, TextSpan?)](../../../Roslynator/CSharp/SyntaxExtensions/RemoveTrivia/README.md) | Creates a new node with the trivia removed\. |
| [RemoveWhitespace\<TNode\>(TNode, TextSpan?)](../../../Roslynator/CSharp/SyntaxExtensions/RemoveWhitespace/README.md) | Creates a new node with the whitespace removed\. |
| [ReplaceWhitespace\<TNode\>(TNode, SyntaxTrivia, TextSpan?)](../../../Roslynator/CSharp/SyntaxExtensions/ReplaceWhitespace/README.md) | Creates a new node with the whitespace replaced\. |
| [SpanContainsDirectives(SyntaxNode)](../../../Roslynator/SyntaxExtensions/SpanContainsDirectives/README.md) | Returns true if the node's span contains any preprocessor directives\. |
| [TrimLeadingTrivia\<TNode\>(TNode)](../../../Roslynator/CSharp/SyntaxExtensions/TrimLeadingTrivia/README.md#1018285907) | Removes all leading whitespace from the leading trivia and returns a new node with the new leading trivia\. [SyntaxKind.WhitespaceTrivia](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.csharp.syntaxkind.whitespacetrivia) and [SyntaxKind.EndOfLineTrivia](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.csharp.syntaxkind.endoflinetrivia) is considered to be a whitespace\. Returns the same node if there is nothing to trim\. |
| [TrimTrailingTrivia\<TNode\>(TNode)](../../../Roslynator/CSharp/SyntaxExtensions/TrimTrailingTrivia/README.md#2450702114) | Removes all trailing whitespace from the trailing trivia and returns a new node with the new trailing trivia\. [SyntaxKind.WhitespaceTrivia](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.csharp.syntaxkind.whitespacetrivia) and [SyntaxKind.EndOfLineTrivia](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.csharp.syntaxkind.endoflinetrivia) is considered to be a whitespace\. Returns the same node if there is nothing to trim\. |
| [TrimTrivia\<TNode\>(TNode)](../../../Roslynator/CSharp/SyntaxExtensions/TrimTrivia/README.md#3568210656) | Removes all leading whitespace from the leading trivia and all trailing whitespace from the trailing trivia and returns a new node with the new trivia\. [SyntaxKind.WhitespaceTrivia](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.csharp.syntaxkind.whitespacetrivia) and [SyntaxKind.EndOfLineTrivia](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.csharp.syntaxkind.endoflinetrivia) is considered to be a whitespace\. Returns the same node if there is nothing to trim\. |
| [WithFormatterAnnotation\<TNode\>(TNode)](../../../Roslynator/CSharp/WorkspaceSyntaxExtensions/WithFormatterAnnotation/README.md#3493763853) | Creates a new node with the [Formatter.Annotation](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.formatting.formatter.annotation) attached\. |
| [WithSimplifierAnnotation\<TNode\>(TNode)](../../../Roslynator/CSharp/WorkspaceSyntaxExtensions/WithSimplifierAnnotation/README.md#1448778190) | Creates a new node with the [Simplifier.Annotation](https://docs.microsoft.com/en-us/dotnet/api/microsoft.codeanalysis.simplification.simplifier.annotation) attached\. |
| [WithTriviaFrom\<TNode\>(TNode, SyntaxToken)](../../../Roslynator/SyntaxExtensions/WithTriviaFrom/README.md#441639473) | Creates a new node from this node with both the leading and trailing trivia of the specified token\. |

