## Generate property for DebuggerDisplay attribute

| Property           | Value                                           |
| ------------------ | ----------------------------------------------- |
| Id                 | RR0204                                          |
| Title              | Generate property for DebuggerDisplay attribute |
| Syntax             | DebuggerDisplay attribute                       |
| Enabled by Default | &#x2713;                                        |

### Usage

#### Before

```csharp
[DebuggerDisplay("A: {A} B: {B}")]
public class Foo
{
    public string A { get; }
    public string B { get; }
}
```

#### After

```csharp
DebuggerDisplay("{DebuggerDisplay,nq}")]
public class Foo
{
    public string A { get; }
    public string B { get; }

    [DebuggerBrowsable(DebuggerBrowsableState.Never)]
    private string DebuggerDisplay
    {
        get { return $"A: {A} B: {B}"; }
    }
}
```

## See Also

* [Using Expressions in DebuggerDisplay](https://docs.microsoft.com/en-us/visualstudio/debugger/using-the-debuggerdisplay-attribute#using-expressions-in-debuggerdisplay)
* [DebuggerDisplay attribute best practices](https://blogs.msdn.microsoft.com/jaredpar/2011/03/18/debuggerdisplay-attribute-best-practices/)
* [Full list of refactorings](Refactorings.md)


*\(Generated with [DotMarkdown](http://github.com/JosefPihrt/DotMarkdown)\)*