﻿// Copyright (c) Josef Pihrt and Contributors. Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

// <auto-generated>

using System;
using System.Diagnostics;
using Microsoft.CodeAnalysis;
using Microsoft.CodeAnalysis.CSharp;
using Microsoft.CodeAnalysis.CSharp.Syntax;

namespace Roslynator.CSharp.SyntaxWalkers
{
    internal abstract class CSharpSyntaxNodeWalker : CSharpSyntaxWalker
    {
        protected CSharpSyntaxNodeWalker() : base(depth: SyntaxWalkerDepth.Node)
        {
        }

        protected virtual bool ShouldVisit
        {
            get
            {
                return true;
            }
        }

        public override void Visit(SyntaxNode node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            base.Visit(node);
        }

        public override void VisitAccessorDeclaration(AccessorDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax body = node.Body;
            if (body != null)
            {
                VisitBlock(body);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitAccessorList(AccessorListSyntax node)
        {
            foreach (AccessorDeclarationSyntax accessorDeclaration in node.Accessors)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAccessorDeclaration(accessorDeclaration);
            }
        }

        public override void VisitAliasQualifiedName(AliasQualifiedNameSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            IdentifierNameSyntax alias = node.Alias;
            if (alias != null)
            {
                VisitIdentifierName(alias);
            }

            if (!ShouldVisit)
            {
                return;
            }

            SimpleNameSyntax name = node.Name;
            if (name != null)
            {
                VisitSimpleName(name);
            }
        }

        public override void VisitAnonymousMethodExpression(AnonymousMethodExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }
        }

        public override void VisitAnonymousObjectCreationExpression(AnonymousObjectCreationExpressionSyntax node)
        {
            foreach (AnonymousObjectMemberDeclaratorSyntax anonymousObjectMemberDeclarator in node.Initializers)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAnonymousObjectMemberDeclarator(anonymousObjectMemberDeclarator);
            }
        }

        public override void VisitAnonymousObjectMemberDeclarator(AnonymousObjectMemberDeclaratorSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameEqualsSyntax nameEquals = node.NameEquals;
            if (nameEquals != null)
            {
                VisitNameEquals(nameEquals);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitArgument(ArgumentSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameColonSyntax nameColon = node.NameColon;
            if (nameColon != null)
            {
                VisitNameColon(nameColon);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitArgumentList(ArgumentListSyntax node)
        {
            foreach (ArgumentSyntax argument in node.Arguments)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitArgument(argument);
            }
        }

        public override void VisitArrayCreationExpression(ArrayCreationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ArrayTypeSyntax type = node.Type;
            if (type != null)
            {
                VisitArrayType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            InitializerExpressionSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitInitializerExpression(initializer);
            }
        }

        public override void VisitArrayRankSpecifier(ArrayRankSpecifierSyntax node)
        {
            foreach (ExpressionSyntax expression in node.Sizes)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExpression(expression);
            }
        }

        public override void VisitArrayType(ArrayTypeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax elementType = node.ElementType;
            if (elementType != null)
            {
                VisitType(elementType);
            }

            foreach (ArrayRankSpecifierSyntax arrayRankSpecifier in node.RankSpecifiers)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitArrayRankSpecifier(arrayRankSpecifier);
            }
        }

        public override void VisitArrowExpressionClause(ArrowExpressionClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitAssignmentExpression(AssignmentExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax left = node.Left;
            if (left != null)
            {
                VisitExpression(left);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax right = node.Right;
            if (right != null)
            {
                VisitExpression(right);
            }
        }

        public override void VisitAttribute(AttributeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameSyntax name = node.Name;
            if (name != null)
            {
                VisitType(name);
            }

            if (!ShouldVisit)
            {
                return;
            }

            AttributeArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitAttributeArgumentList(argumentList);
            }
        }

        public override void VisitAttributeArgument(AttributeArgumentSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameEqualsSyntax nameEquals = node.NameEquals;
            if (nameEquals != null)
            {
                VisitNameEquals(nameEquals);
            }

            if (!ShouldVisit)
            {
                return;
            }

            NameColonSyntax nameColon = node.NameColon;
            if (nameColon != null)
            {
                VisitNameColon(nameColon);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitAttributeArgumentList(AttributeArgumentListSyntax node)
        {
            foreach (AttributeArgumentSyntax attributeArgument in node.Arguments)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeArgument(attributeArgument);
            }
        }

        public override void VisitAttributeList(AttributeListSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            AttributeTargetSpecifierSyntax target = node.Target;
            if (target != null)
            {
                VisitAttributeTargetSpecifier(target);
            }

            foreach (AttributeSyntax attribute in node.Attributes)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttribute(attribute);
            }
        }

        public override void VisitAttributeTargetSpecifier(AttributeTargetSpecifierSyntax node)
        {
        }

        public override void VisitAwaitExpression(AwaitExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitBadDirectiveTrivia(BadDirectiveTriviaSyntax node)
        {
        }

        public override void VisitBaseExpression(BaseExpressionSyntax node)
        {
        }

        public override void VisitBaseList(BaseListSyntax node)
        {
            foreach (BaseTypeSyntax baseType in node.Types)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitBaseType(baseType);
            }
        }

        public override void VisitBinaryExpression(BinaryExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax left = node.Left;
            if (left != null)
            {
                VisitExpression(left);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax right = node.Right;
            if (right != null)
            {
                VisitExpression(right);
            }
        }

        public override void VisitBinaryPattern(BinaryPatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax left = node.Left;
            if (left != null)
            {
                VisitPattern(left);
            }

            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax right = node.Right;
            if (right != null)
            {
                VisitPattern(right);
            }
        }

        public override void VisitBlock(BlockSyntax node)
        {
            foreach (StatementSyntax statement in node.Statements)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitStatement(statement);
            }
        }

        public override void VisitBracketedArgumentList(BracketedArgumentListSyntax node)
        {
            foreach (ArgumentSyntax argument in node.Arguments)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitArgument(argument);
            }
        }

        public override void VisitBracketedParameterList(BracketedParameterListSyntax node)
        {
            foreach (ParameterSyntax parameter in node.Parameters)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitParameter(parameter);
            }
        }

        public override void VisitBreakStatement(BreakStatementSyntax node)
        {
        }

        public override void VisitCasePatternSwitchLabel(CasePatternSwitchLabelSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax pattern = node.Pattern;
            if (pattern != null)
            {
                VisitPattern(pattern);
            }

            if (!ShouldVisit)
            {
                return;
            }

            WhenClauseSyntax whenClause = node.WhenClause;
            if (whenClause != null)
            {
                VisitWhenClause(whenClause);
            }
        }

        public override void VisitCaseSwitchLabel(CaseSwitchLabelSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax value = node.Value;
            if (value != null)
            {
                VisitExpression(value);
            }
        }

        public override void VisitCastExpression(CastExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitCatchClause(CatchClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            CatchDeclarationSyntax declaration = node.Declaration;
            if (declaration != null)
            {
                VisitCatchDeclaration(declaration);
            }

            if (!ShouldVisit)
            {
                return;
            }

            CatchFilterClauseSyntax filter = node.Filter;
            if (filter != null)
            {
                VisitCatchFilterClause(filter);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }
        }

        public override void VisitCatchDeclaration(CatchDeclarationSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitCatchFilterClause(CatchFilterClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax filterExpression = node.FilterExpression;
            if (filterExpression != null)
            {
                VisitExpression(filterExpression);
            }
        }

        public override void VisitClassDeclaration(ClassDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeParameterListSyntax typeParameterList = node.TypeParameterList;
            if (typeParameterList != null)
            {
                VisitTypeParameterList(typeParameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BaseListSyntax baseList = node.BaseList;
            if (baseList != null)
            {
                VisitBaseList(baseList);
            }

            foreach (TypeParameterConstraintClauseSyntax typeParameterConstraintClause in node.ConstraintClauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraintClause(typeParameterConstraintClause);
            }

            foreach (MemberDeclarationSyntax memberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitMemberDeclaration(memberDeclaration);
            }
        }

        public override void VisitClassOrStructConstraint(ClassOrStructConstraintSyntax node)
        {
        }

        public override void VisitCompilationUnit(CompilationUnitSyntax node)
        {
            foreach (ExternAliasDirectiveSyntax externAliasDirective in node.Externs)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExternAliasDirective(externAliasDirective);
            }

            foreach (UsingDirectiveSyntax usingDirective in node.Usings)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitUsingDirective(usingDirective);
            }

            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            foreach (MemberDeclarationSyntax memberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitMemberDeclaration(memberDeclaration);
            }
        }

        public override void VisitConditionalAccessExpression(ConditionalAccessExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax whenNotNull = node.WhenNotNull;
            if (whenNotNull != null)
            {
                VisitExpression(whenNotNull);
            }
        }

        public override void VisitConditionalExpression(ConditionalExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax whenTrue = node.WhenTrue;
            if (whenTrue != null)
            {
                VisitExpression(whenTrue);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax whenFalse = node.WhenFalse;
            if (whenFalse != null)
            {
                VisitExpression(whenFalse);
            }
        }

        public override void VisitConstantPattern(ConstantPatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitConstructorConstraint(ConstructorConstraintSyntax node)
        {
        }

        public override void VisitConstructorDeclaration(ConstructorDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ConstructorInitializerSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitConstructorInitializer(initializer);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax body = node.Body;
            if (body != null)
            {
                VisitBlock(body);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitConstructorInitializer(ConstructorInitializerSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitArgumentList(argumentList);
            }
        }

        public override void VisitContinueStatement(ContinueStatementSyntax node)
        {
        }

        public override void VisitConversionOperatorDeclaration(ConversionOperatorDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExplicitInterfaceSpecifierSyntax explicitInterfaceSpecifier = node.ExplicitInterfaceSpecifier;
            if (explicitInterfaceSpecifier != null)
            {
                VisitExplicitInterfaceSpecifier(explicitInterfaceSpecifier);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax body = node.Body;
            if (body != null)
            {
                VisitBlock(body);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitConversionOperatorMemberCref(ConversionOperatorMemberCrefSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            CrefParameterListSyntax parameters = node.Parameters;
            if (parameters != null)
            {
                VisitCrefParameterList(parameters);
            }
        }

        public override void VisitCrefBracketedParameterList(CrefBracketedParameterListSyntax node)
        {
            foreach (CrefParameterSyntax crefParameter in node.Parameters)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitCrefParameter(crefParameter);
            }
        }

        public override void VisitCrefParameter(CrefParameterSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitCrefParameterList(CrefParameterListSyntax node)
        {
            foreach (CrefParameterSyntax crefParameter in node.Parameters)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitCrefParameter(crefParameter);
            }
        }

        public override void VisitDeclarationExpression(DeclarationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            VariableDesignationSyntax designation = node.Designation;
            if (designation != null)
            {
                VisitVariableDesignation(designation);
            }
        }

        public override void VisitDeclarationPattern(DeclarationPatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            VariableDesignationSyntax designation = node.Designation;
            if (designation != null)
            {
                VisitVariableDesignation(designation);
            }
        }

        public override void VisitDefaultConstraint(DefaultConstraintSyntax node)
        {
        }

        public override void VisitDefaultExpression(DefaultExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitDefaultSwitchLabel(DefaultSwitchLabelSyntax node)
        {
        }

        public override void VisitDefineDirectiveTrivia(DefineDirectiveTriviaSyntax node)
        {
        }

        public override void VisitDelegateDeclaration(DelegateDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax returnType = node.ReturnType;
            if (returnType != null)
            {
                VisitType(returnType);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeParameterListSyntax typeParameterList = node.TypeParameterList;
            if (typeParameterList != null)
            {
                VisitTypeParameterList(typeParameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            foreach (TypeParameterConstraintClauseSyntax typeParameterConstraintClause in node.ConstraintClauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraintClause(typeParameterConstraintClause);
            }
        }

        public override void VisitDestructorDeclaration(DestructorDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax body = node.Body;
            if (body != null)
            {
                VisitBlock(body);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitDiscardDesignation(DiscardDesignationSyntax node)
        {
        }

        public override void VisitDiscardPattern(DiscardPatternSyntax node)
        {
        }

        public override void VisitDocumentationCommentTrivia(DocumentationCommentTriviaSyntax node)
        {
            foreach (XmlNodeSyntax xmlNode in node.Content)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitXmlNode(xmlNode);
            }
        }

        public override void VisitDoStatement(DoStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }
        }

        public override void VisitElementAccessExpression(ElementAccessExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BracketedArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitBracketedArgumentList(argumentList);
            }
        }

        public override void VisitElementBindingExpression(ElementBindingExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            BracketedArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitBracketedArgumentList(argumentList);
            }
        }

        public override void VisitElifDirectiveTrivia(ElifDirectiveTriviaSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }
        }

        public override void VisitElseClause(ElseClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitElseDirectiveTrivia(ElseDirectiveTriviaSyntax node)
        {
        }

        public override void VisitEmptyStatement(EmptyStatementSyntax node)
        {
        }

        public override void VisitEndIfDirectiveTrivia(EndIfDirectiveTriviaSyntax node)
        {
        }

        public override void VisitEndRegionDirectiveTrivia(EndRegionDirectiveTriviaSyntax node)
        {
        }

        public override void VisitEnumDeclaration(EnumDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BaseListSyntax baseList = node.BaseList;
            if (baseList != null)
            {
                VisitBaseList(baseList);
            }

            foreach (EnumMemberDeclarationSyntax enumMemberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitEnumMemberDeclaration(enumMemberDeclaration);
            }
        }

        public override void VisitEnumMemberDeclaration(EnumMemberDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            EqualsValueClauseSyntax equalsValue = node.EqualsValue;
            if (equalsValue != null)
            {
                VisitEqualsValueClause(equalsValue);
            }
        }

        public override void VisitEqualsValueClause(EqualsValueClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax value = node.Value;
            if (value != null)
            {
                VisitExpression(value);
            }
        }

        public override void VisitErrorDirectiveTrivia(ErrorDirectiveTriviaSyntax node)
        {
        }

        public override void VisitEventDeclaration(EventDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExplicitInterfaceSpecifierSyntax explicitInterfaceSpecifier = node.ExplicitInterfaceSpecifier;
            if (explicitInterfaceSpecifier != null)
            {
                VisitExplicitInterfaceSpecifier(explicitInterfaceSpecifier);
            }

            if (!ShouldVisit)
            {
                return;
            }

            AccessorListSyntax accessorList = node.AccessorList;
            if (accessorList != null)
            {
                VisitAccessorList(accessorList);
            }
        }

        public override void VisitEventFieldDeclaration(EventFieldDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            VariableDeclarationSyntax declaration = node.Declaration;
            if (declaration != null)
            {
                VisitVariableDeclaration(declaration);
            }
        }

        public override void VisitExplicitInterfaceSpecifier(ExplicitInterfaceSpecifierSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameSyntax name = node.Name;
            if (name != null)
            {
                VisitType(name);
            }
        }

        public override void VisitExpressionColon(ExpressionColonSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitExpressionStatement(ExpressionStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitExternAliasDirective(ExternAliasDirectiveSyntax node)
        {
        }

        public override void VisitFieldDeclaration(FieldDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            VariableDeclarationSyntax declaration = node.Declaration;
            if (declaration != null)
            {
                VisitVariableDeclaration(declaration);
            }
        }

        public override void VisitFileScopedNamespaceDeclaration(FileScopedNamespaceDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            NameSyntax name = node.Name;
            if (name != null)
            {
                VisitType(name);
            }

            foreach (ExternAliasDirectiveSyntax externAliasDirective in node.Externs)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExternAliasDirective(externAliasDirective);
            }

            foreach (UsingDirectiveSyntax usingDirective in node.Usings)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitUsingDirective(usingDirective);
            }

            foreach (MemberDeclarationSyntax memberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitMemberDeclaration(memberDeclaration);
            }
        }

        public override void VisitFinallyClause(FinallyClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }
        }

        public override void VisitFixedStatement(FixedStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            VariableDeclarationSyntax declaration = node.Declaration;
            if (declaration != null)
            {
                VisitVariableDeclaration(declaration);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitForEachStatement(ForEachStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitForEachVariableStatement(ForEachVariableStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax variable = node.Variable;
            if (variable != null)
            {
                VisitExpression(variable);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitForStatement(ForStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            VariableDeclarationSyntax declaration = node.Declaration;
            if (declaration != null)
            {
                VisitVariableDeclaration(declaration);
            }

            foreach (ExpressionSyntax expression in node.Initializers)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }

            foreach (ExpressionSyntax expression2 in node.Incrementors)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExpression(expression2);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitFromClause(FromClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitFunctionPointerCallingConvention(FunctionPointerCallingConventionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            FunctionPointerUnmanagedCallingConventionListSyntax unmanagedCallingConventionList = node.UnmanagedCallingConventionList;
            if (unmanagedCallingConventionList != null)
            {
                VisitFunctionPointerUnmanagedCallingConventionList(unmanagedCallingConventionList);
            }
        }

        public override void VisitFunctionPointerParameter(FunctionPointerParameterSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitFunctionPointerParameterList(FunctionPointerParameterListSyntax node)
        {
            foreach (FunctionPointerParameterSyntax functionPointerParameter in node.Parameters)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitFunctionPointerParameter(functionPointerParameter);
            }
        }

        public override void VisitFunctionPointerType(FunctionPointerTypeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            FunctionPointerCallingConventionSyntax callingConvention = node.CallingConvention;
            if (callingConvention != null)
            {
                VisitFunctionPointerCallingConvention(callingConvention);
            }

            if (!ShouldVisit)
            {
                return;
            }

            FunctionPointerParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitFunctionPointerParameterList(parameterList);
            }
        }

        public override void VisitFunctionPointerUnmanagedCallingConvention(FunctionPointerUnmanagedCallingConventionSyntax node)
        {
        }

        public override void VisitFunctionPointerUnmanagedCallingConventionList(FunctionPointerUnmanagedCallingConventionListSyntax node)
        {
            foreach (FunctionPointerUnmanagedCallingConventionSyntax functionPointerUnmanagedCallingConvention in node.CallingConventions)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitFunctionPointerUnmanagedCallingConvention(functionPointerUnmanagedCallingConvention);
            }
        }

        public override void VisitGenericName(GenericNameSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeArgumentListSyntax typeArgumentList = node.TypeArgumentList;
            if (typeArgumentList != null)
            {
                VisitTypeArgumentList(typeArgumentList);
            }
        }

        public override void VisitGlobalStatement(GlobalStatementSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitGotoStatement(GotoStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitGroupClause(GroupClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax groupExpression = node.GroupExpression;
            if (groupExpression != null)
            {
                VisitExpression(groupExpression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax byExpression = node.ByExpression;
            if (byExpression != null)
            {
                VisitExpression(byExpression);
            }
        }

        public override void VisitCheckedExpression(CheckedExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitCheckedStatement(CheckedStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }
        }

        public override void VisitIdentifierName(IdentifierNameSyntax node)
        {
        }

        public override void VisitIfDirectiveTrivia(IfDirectiveTriviaSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }
        }

        public override void VisitIfStatement(IfStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ElseClauseSyntax @else = node.Else;
            if (@else != null)
            {
                VisitElseClause(@else);
            }
        }

        public override void VisitImplicitArrayCreationExpression(ImplicitArrayCreationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            InitializerExpressionSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitInitializerExpression(initializer);
            }
        }

        public override void VisitImplicitElementAccess(ImplicitElementAccessSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            BracketedArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitBracketedArgumentList(argumentList);
            }
        }

        public override void VisitImplicitObjectCreationExpression(ImplicitObjectCreationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitArgumentList(argumentList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            InitializerExpressionSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitInitializerExpression(initializer);
            }
        }

        public override void VisitImplicitStackAllocArrayCreationExpression(ImplicitStackAllocArrayCreationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            InitializerExpressionSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitInitializerExpression(initializer);
            }
        }

        public override void VisitIncompleteMember(IncompleteMemberSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitIndexerDeclaration(IndexerDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExplicitInterfaceSpecifierSyntax explicitInterfaceSpecifier = node.ExplicitInterfaceSpecifier;
            if (explicitInterfaceSpecifier != null)
            {
                VisitExplicitInterfaceSpecifier(explicitInterfaceSpecifier);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BracketedParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitBracketedParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            AccessorListSyntax accessorList = node.AccessorList;
            if (accessorList != null)
            {
                VisitAccessorList(accessorList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitIndexerMemberCref(IndexerMemberCrefSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            CrefBracketedParameterListSyntax parameters = node.Parameters;
            if (parameters != null)
            {
                VisitCrefBracketedParameterList(parameters);
            }
        }

        public override void VisitInitializerExpression(InitializerExpressionSyntax node)
        {
            foreach (ExpressionSyntax expression in node.Expressions)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExpression(expression);
            }
        }

        public override void VisitInterfaceDeclaration(InterfaceDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeParameterListSyntax typeParameterList = node.TypeParameterList;
            if (typeParameterList != null)
            {
                VisitTypeParameterList(typeParameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BaseListSyntax baseList = node.BaseList;
            if (baseList != null)
            {
                VisitBaseList(baseList);
            }

            foreach (TypeParameterConstraintClauseSyntax typeParameterConstraintClause in node.ConstraintClauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraintClause(typeParameterConstraintClause);
            }

            foreach (MemberDeclarationSyntax memberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitMemberDeclaration(memberDeclaration);
            }
        }

        public override void VisitInterpolatedStringExpression(InterpolatedStringExpressionSyntax node)
        {
            foreach (InterpolatedStringContentSyntax interpolatedStringContent in node.Contents)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitInterpolatedStringContent(interpolatedStringContent);
            }
        }

        public override void VisitInterpolatedStringText(InterpolatedStringTextSyntax node)
        {
        }

        public override void VisitInterpolation(InterpolationSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            InterpolationAlignmentClauseSyntax alignmentClause = node.AlignmentClause;
            if (alignmentClause != null)
            {
                VisitInterpolationAlignmentClause(alignmentClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            InterpolationFormatClauseSyntax formatClause = node.FormatClause;
            if (formatClause != null)
            {
                VisitInterpolationFormatClause(formatClause);
            }
        }

        public override void VisitInterpolationAlignmentClause(InterpolationAlignmentClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax value = node.Value;
            if (value != null)
            {
                VisitExpression(value);
            }
        }

        public override void VisitInterpolationFormatClause(InterpolationFormatClauseSyntax node)
        {
        }

        public override void VisitInvocationExpression(InvocationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitArgumentList(argumentList);
            }
        }

        public override void VisitIsPatternExpression(IsPatternExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax pattern = node.Pattern;
            if (pattern != null)
            {
                VisitPattern(pattern);
            }
        }

        public override void VisitJoinClause(JoinClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax inExpression = node.InExpression;
            if (inExpression != null)
            {
                VisitExpression(inExpression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax leftExpression = node.LeftExpression;
            if (leftExpression != null)
            {
                VisitExpression(leftExpression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax rightExpression = node.RightExpression;
            if (rightExpression != null)
            {
                VisitExpression(rightExpression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            JoinIntoClauseSyntax into = node.Into;
            if (into != null)
            {
                VisitJoinIntoClause(into);
            }
        }

        public override void VisitJoinIntoClause(JoinIntoClauseSyntax node)
        {
        }

        public override void VisitLabeledStatement(LabeledStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitLetClause(LetClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitLineDirectivePosition(LineDirectivePositionSyntax node)
        {
        }

        public override void VisitLineDirectiveTrivia(LineDirectiveTriviaSyntax node)
        {
        }

        public override void VisitLineSpanDirectiveTrivia(LineSpanDirectiveTriviaSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            LineDirectivePositionSyntax start = node.Start;
            if (start != null)
            {
                VisitLineDirectivePosition(start);
            }

            if (!ShouldVisit)
            {
                return;
            }

            LineDirectivePositionSyntax end = node.End;
            if (end != null)
            {
                VisitLineDirectivePosition(end);
            }
        }

        public override void VisitLiteralExpression(LiteralExpressionSyntax node)
        {
        }

        public override void VisitLoadDirectiveTrivia(LoadDirectiveTriviaSyntax node)
        {
        }

        public override void VisitLocalDeclarationStatement(LocalDeclarationStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            VariableDeclarationSyntax declaration = node.Declaration;
            if (declaration != null)
            {
                VisitVariableDeclaration(declaration);
            }
        }

        public override void VisitLocalFunctionStatement(LocalFunctionStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax returnType = node.ReturnType;
            if (returnType != null)
            {
                VisitType(returnType);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeParameterListSyntax typeParameterList = node.TypeParameterList;
            if (typeParameterList != null)
            {
                VisitTypeParameterList(typeParameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            foreach (TypeParameterConstraintClauseSyntax typeParameterConstraintClause in node.ConstraintClauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraintClause(typeParameterConstraintClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax body = node.Body;
            if (body != null)
            {
                VisitBlock(body);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitLockStatement(LockStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitMakeRefExpression(MakeRefExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitMemberAccessExpression(MemberAccessExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            SimpleNameSyntax name = node.Name;
            if (name != null)
            {
                VisitSimpleName(name);
            }
        }

        public override void VisitMemberBindingExpression(MemberBindingExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            SimpleNameSyntax name = node.Name;
            if (name != null)
            {
                VisitSimpleName(name);
            }
        }

        public override void VisitMethodDeclaration(MethodDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax returnType = node.ReturnType;
            if (returnType != null)
            {
                VisitType(returnType);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExplicitInterfaceSpecifierSyntax explicitInterfaceSpecifier = node.ExplicitInterfaceSpecifier;
            if (explicitInterfaceSpecifier != null)
            {
                VisitExplicitInterfaceSpecifier(explicitInterfaceSpecifier);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeParameterListSyntax typeParameterList = node.TypeParameterList;
            if (typeParameterList != null)
            {
                VisitTypeParameterList(typeParameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            foreach (TypeParameterConstraintClauseSyntax typeParameterConstraintClause in node.ConstraintClauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraintClause(typeParameterConstraintClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax body = node.Body;
            if (body != null)
            {
                VisitBlock(body);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitNameColon(NameColonSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            IdentifierNameSyntax name = node.Name;
            if (name != null)
            {
                VisitIdentifierName(name);
            }
        }

        public override void VisitNameEquals(NameEqualsSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            IdentifierNameSyntax name = node.Name;
            if (name != null)
            {
                VisitIdentifierName(name);
            }
        }

        public override void VisitNameMemberCref(NameMemberCrefSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax name = node.Name;
            if (name != null)
            {
                VisitType(name);
            }

            if (!ShouldVisit)
            {
                return;
            }

            CrefParameterListSyntax parameters = node.Parameters;
            if (parameters != null)
            {
                VisitCrefParameterList(parameters);
            }
        }

        public override void VisitNamespaceDeclaration(NamespaceDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            NameSyntax name = node.Name;
            if (name != null)
            {
                VisitType(name);
            }

            foreach (ExternAliasDirectiveSyntax externAliasDirective in node.Externs)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExternAliasDirective(externAliasDirective);
            }

            foreach (UsingDirectiveSyntax usingDirective in node.Usings)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitUsingDirective(usingDirective);
            }

            foreach (MemberDeclarationSyntax memberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitMemberDeclaration(memberDeclaration);
            }
        }

        public override void VisitNullableDirectiveTrivia(NullableDirectiveTriviaSyntax node)
        {
        }

        public override void VisitNullableType(NullableTypeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax elementType = node.ElementType;
            if (elementType != null)
            {
                VisitType(elementType);
            }
        }

        public override void VisitObjectCreationExpression(ObjectCreationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitArgumentList(argumentList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            InitializerExpressionSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitInitializerExpression(initializer);
            }
        }

        public override void VisitOmittedArraySizeExpression(OmittedArraySizeExpressionSyntax node)
        {
        }

        public override void VisitOmittedTypeArgument(OmittedTypeArgumentSyntax node)
        {
        }

        public override void VisitOperatorDeclaration(OperatorDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax returnType = node.ReturnType;
            if (returnType != null)
            {
                VisitType(returnType);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExplicitInterfaceSpecifierSyntax explicitInterfaceSpecifier = node.ExplicitInterfaceSpecifier;
            if (explicitInterfaceSpecifier != null)
            {
                VisitExplicitInterfaceSpecifier(explicitInterfaceSpecifier);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax body = node.Body;
            if (body != null)
            {
                VisitBlock(body);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }
        }

        public override void VisitOperatorMemberCref(OperatorMemberCrefSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            CrefParameterListSyntax parameters = node.Parameters;
            if (parameters != null)
            {
                VisitCrefParameterList(parameters);
            }
        }

        public override void VisitOrderByClause(OrderByClauseSyntax node)
        {
            foreach (OrderingSyntax ordering in node.Orderings)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitOrdering(ordering);
            }
        }

        public override void VisitOrdering(OrderingSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitParameter(ParameterSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            EqualsValueClauseSyntax @default = node.Default;
            if (@default != null)
            {
                VisitEqualsValueClause(@default);
            }
        }

        public override void VisitParameterList(ParameterListSyntax node)
        {
            foreach (ParameterSyntax parameter in node.Parameters)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitParameter(parameter);
            }
        }

        public override void VisitParenthesizedExpression(ParenthesizedExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitParenthesizedLambdaExpression(ParenthesizedLambdaExpressionSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax returnType = node.ReturnType;
            if (returnType != null)
            {
                VisitType(returnType);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitExpression(expressionBody);
            }
        }

        public override void VisitParenthesizedPattern(ParenthesizedPatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax pattern = node.Pattern;
            if (pattern != null)
            {
                VisitPattern(pattern);
            }
        }

        public override void VisitParenthesizedVariableDesignation(ParenthesizedVariableDesignationSyntax node)
        {
            foreach (VariableDesignationSyntax variableDesignation in node.Variables)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitVariableDesignation(variableDesignation);
            }
        }

        public override void VisitPointerType(PointerTypeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax elementType = node.ElementType;
            if (elementType != null)
            {
                VisitType(elementType);
            }
        }

        public override void VisitPositionalPatternClause(PositionalPatternClauseSyntax node)
        {
            foreach (SubpatternSyntax subpattern in node.Subpatterns)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitSubpattern(subpattern);
            }
        }

        public override void VisitPostfixUnaryExpression(PostfixUnaryExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax operand = node.Operand;
            if (operand != null)
            {
                VisitExpression(operand);
            }
        }

        public override void VisitPragmaChecksumDirectiveTrivia(PragmaChecksumDirectiveTriviaSyntax node)
        {
        }

        public override void VisitPragmaWarningDirectiveTrivia(PragmaWarningDirectiveTriviaSyntax node)
        {
            foreach (ExpressionSyntax expression in node.ErrorCodes)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitExpression(expression);
            }
        }

        public override void VisitPredefinedType(PredefinedTypeSyntax node)
        {
        }

        public override void VisitPrefixUnaryExpression(PrefixUnaryExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax operand = node.Operand;
            if (operand != null)
            {
                VisitExpression(operand);
            }
        }

        public override void VisitPrimaryConstructorBaseType(PrimaryConstructorBaseTypeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitArgumentList(argumentList);
            }
        }

        public override void VisitPropertyDeclaration(PropertyDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExplicitInterfaceSpecifierSyntax explicitInterfaceSpecifier = node.ExplicitInterfaceSpecifier;
            if (explicitInterfaceSpecifier != null)
            {
                VisitExplicitInterfaceSpecifier(explicitInterfaceSpecifier);
            }

            if (!ShouldVisit)
            {
                return;
            }

            AccessorListSyntax accessorList = node.AccessorList;
            if (accessorList != null)
            {
                VisitAccessorList(accessorList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ArrowExpressionClauseSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitArrowExpressionClause(expressionBody);
            }

            if (!ShouldVisit)
            {
                return;
            }

            EqualsValueClauseSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitEqualsValueClause(initializer);
            }
        }

        public override void VisitPropertyPatternClause(PropertyPatternClauseSyntax node)
        {
            foreach (SubpatternSyntax subpattern in node.Subpatterns)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitSubpattern(subpattern);
            }
        }

        public override void VisitQualifiedCref(QualifiedCrefSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax container = node.Container;
            if (container != null)
            {
                VisitType(container);
            }

            if (!ShouldVisit)
            {
                return;
            }

            MemberCrefSyntax member = node.Member;
            if (member != null)
            {
                VisitMemberCref(member);
            }
        }

        public override void VisitQualifiedName(QualifiedNameSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameSyntax left = node.Left;
            if (left != null)
            {
                VisitType(left);
            }

            if (!ShouldVisit)
            {
                return;
            }

            SimpleNameSyntax right = node.Right;
            if (right != null)
            {
                VisitSimpleName(right);
            }
        }

        public override void VisitQueryBody(QueryBodySyntax node)
        {
            foreach (QueryClauseSyntax queryClause in node.Clauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitQueryClause(queryClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            SelectOrGroupClauseSyntax selectOrGroup = node.SelectOrGroup;
            if (selectOrGroup != null)
            {
                VisitSelectOrGroupClause(selectOrGroup);
            }

            if (!ShouldVisit)
            {
                return;
            }

            QueryContinuationSyntax continuation = node.Continuation;
            if (continuation != null)
            {
                VisitQueryContinuation(continuation);
            }
        }

        public override void VisitQueryContinuation(QueryContinuationSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            QueryBodySyntax body = node.Body;
            if (body != null)
            {
                VisitQueryBody(body);
            }
        }

        public override void VisitQueryExpression(QueryExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            FromClauseSyntax fromClause = node.FromClause;
            if (fromClause != null)
            {
                VisitFromClause(fromClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            QueryBodySyntax body = node.Body;
            if (body != null)
            {
                VisitQueryBody(body);
            }
        }

        public override void VisitRangeExpression(RangeExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax leftOperand = node.LeftOperand;
            if (leftOperand != null)
            {
                VisitExpression(leftOperand);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax rightOperand = node.RightOperand;
            if (rightOperand != null)
            {
                VisitExpression(rightOperand);
            }
        }

        public override void VisitRecordDeclaration(RecordDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeParameterListSyntax typeParameterList = node.TypeParameterList;
            if (typeParameterList != null)
            {
                VisitTypeParameterList(typeParameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterListSyntax parameterList = node.ParameterList;
            if (parameterList != null)
            {
                VisitParameterList(parameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BaseListSyntax baseList = node.BaseList;
            if (baseList != null)
            {
                VisitBaseList(baseList);
            }

            foreach (TypeParameterConstraintClauseSyntax typeParameterConstraintClause in node.ConstraintClauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraintClause(typeParameterConstraintClause);
            }

            foreach (MemberDeclarationSyntax memberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitMemberDeclaration(memberDeclaration);
            }
        }

        public override void VisitRecursivePattern(RecursivePatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            PositionalPatternClauseSyntax positionalPatternClause = node.PositionalPatternClause;
            if (positionalPatternClause != null)
            {
                VisitPositionalPatternClause(positionalPatternClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            PropertyPatternClauseSyntax propertyPatternClause = node.PropertyPatternClause;
            if (propertyPatternClause != null)
            {
                VisitPropertyPatternClause(propertyPatternClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            VariableDesignationSyntax designation = node.Designation;
            if (designation != null)
            {
                VisitVariableDesignation(designation);
            }
        }

        public override void VisitReferenceDirectiveTrivia(ReferenceDirectiveTriviaSyntax node)
        {
        }

        public override void VisitRefExpression(RefExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitRefType(RefTypeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitRefTypeExpression(RefTypeExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitRefValueExpression(RefValueExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitRegionDirectiveTrivia(RegionDirectiveTriviaSyntax node)
        {
        }

        public override void VisitRelationalPattern(RelationalPatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitReturnStatement(ReturnStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitSelectClause(SelectClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitShebangDirectiveTrivia(ShebangDirectiveTriviaSyntax node)
        {
        }

        public override void VisitSimpleBaseType(SimpleBaseTypeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitSimpleLambdaExpression(SimpleLambdaExpressionSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ParameterSyntax parameter = node.Parameter;
            if (parameter != null)
            {
                VisitParameter(parameter);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expressionBody = node.ExpressionBody;
            if (expressionBody != null)
            {
                VisitExpression(expressionBody);
            }
        }

        public override void VisitSingleVariableDesignation(SingleVariableDesignationSyntax node)
        {
        }

        public override void VisitSizeOfExpression(SizeOfExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitSkippedTokensTrivia(SkippedTokensTriviaSyntax node)
        {
        }

        public override void VisitStackAllocArrayCreationExpression(StackAllocArrayCreationExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            if (!ShouldVisit)
            {
                return;
            }

            InitializerExpressionSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitInitializerExpression(initializer);
            }
        }

        public override void VisitStructDeclaration(StructDeclarationSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            TypeParameterListSyntax typeParameterList = node.TypeParameterList;
            if (typeParameterList != null)
            {
                VisitTypeParameterList(typeParameterList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BaseListSyntax baseList = node.BaseList;
            if (baseList != null)
            {
                VisitBaseList(baseList);
            }

            foreach (TypeParameterConstraintClauseSyntax typeParameterConstraintClause in node.ConstraintClauses)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraintClause(typeParameterConstraintClause);
            }

            foreach (MemberDeclarationSyntax memberDeclaration in node.Members)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitMemberDeclaration(memberDeclaration);
            }
        }

        public override void VisitSubpattern(SubpatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameColonSyntax nameColon = node.NameColon;
            if (nameColon != null)
            {
                VisitNameColon(nameColon);
            }

            if (!ShouldVisit)
            {
                return;
            }

            BaseExpressionColonSyntax expressionColon = node.ExpressionColon;
            if (expressionColon != null)
            {
                VisitBaseExpressionColon(expressionColon);
            }

            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax pattern = node.Pattern;
            if (pattern != null)
            {
                VisitPattern(pattern);
            }
        }

        public override void VisitSwitchExpression(SwitchExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax governingExpression = node.GoverningExpression;
            if (governingExpression != null)
            {
                VisitExpression(governingExpression);
            }

            foreach (SwitchExpressionArmSyntax switchExpressionArm in node.Arms)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitSwitchExpressionArm(switchExpressionArm);
            }
        }

        public override void VisitSwitchExpressionArm(SwitchExpressionArmSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax pattern = node.Pattern;
            if (pattern != null)
            {
                VisitPattern(pattern);
            }

            if (!ShouldVisit)
            {
                return;
            }

            WhenClauseSyntax whenClause = node.WhenClause;
            if (whenClause != null)
            {
                VisitWhenClause(whenClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitSwitchSection(SwitchSectionSyntax node)
        {
            foreach (SwitchLabelSyntax switchLabel in node.Labels)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitSwitchLabel(switchLabel);
            }

            foreach (StatementSyntax statement in node.Statements)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitStatement(statement);
            }
        }

        public override void VisitSwitchStatement(SwitchStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            foreach (SwitchSectionSyntax switchSection in node.Sections)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitSwitchSection(switchSection);
            }
        }

        public override void VisitThisExpression(ThisExpressionSyntax node)
        {
        }

        public override void VisitThrowExpression(ThrowExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitThrowStatement(ThrowStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        public override void VisitTryStatement(TryStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }

            foreach (CatchClauseSyntax catchClause in node.Catches)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitCatchClause(catchClause);
            }

            if (!ShouldVisit)
            {
                return;
            }

            FinallyClauseSyntax @finally = node.Finally;
            if (@finally != null)
            {
                VisitFinallyClause(@finally);
            }
        }

        public override void VisitTupleElement(TupleElementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitTupleExpression(TupleExpressionSyntax node)
        {
            foreach (ArgumentSyntax argument in node.Arguments)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitArgument(argument);
            }
        }

        public override void VisitTupleType(TupleTypeSyntax node)
        {
            foreach (TupleElementSyntax tupleElement in node.Elements)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTupleElement(tupleElement);
            }
        }

        public override void VisitTypeArgumentList(TypeArgumentListSyntax node)
        {
            foreach (TypeSyntax type in node.Arguments)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitType(type);
            }
        }

        public override void VisitTypeConstraint(TypeConstraintSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitTypeCref(TypeCrefSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitTypeOfExpression(TypeOfExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitTypeParameter(TypeParameterSyntax node)
        {
            foreach (AttributeListSyntax attributeList in node.AttributeLists)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitAttributeList(attributeList);
            }
        }

        public override void VisitTypeParameterConstraintClause(TypeParameterConstraintClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            IdentifierNameSyntax name = node.Name;
            if (name != null)
            {
                VisitIdentifierName(name);
            }

            foreach (TypeParameterConstraintSyntax typeParameterConstraint in node.Constraints)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameterConstraint(typeParameterConstraint);
            }
        }

        public override void VisitTypeParameterList(TypeParameterListSyntax node)
        {
            foreach (TypeParameterSyntax typeParameter in node.Parameters)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitTypeParameter(typeParameter);
            }
        }

        public override void VisitTypePattern(TypePatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }
        }

        public override void VisitUnaryPattern(UnaryPatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            PatternSyntax pattern = node.Pattern;
            if (pattern != null)
            {
                VisitPattern(pattern);
            }
        }

        public override void VisitUndefDirectiveTrivia(UndefDirectiveTriviaSyntax node)
        {
        }

        public override void VisitUnsafeStatement(UnsafeStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            BlockSyntax block = node.Block;
            if (block != null)
            {
                VisitBlock(block);
            }
        }

        public override void VisitUsingDirective(UsingDirectiveSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            NameEqualsSyntax alias = node.Alias;
            if (alias != null)
            {
                VisitNameEquals(alias);
            }

            if (!ShouldVisit)
            {
                return;
            }

            NameSyntax name = node.Name;
            if (name != null)
            {
                VisitType(name);
            }
        }

        public override void VisitUsingStatement(UsingStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            VariableDeclarationSyntax declaration = node.Declaration;
            if (declaration != null)
            {
                VisitVariableDeclaration(declaration);
            }

            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitVariableDeclaration(VariableDeclarationSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            TypeSyntax type = node.Type;
            if (type != null)
            {
                VisitType(type);
            }

            foreach (VariableDeclaratorSyntax variableDeclarator in node.Variables)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitVariableDeclarator(variableDeclarator);
            }
        }

        public override void VisitVariableDeclarator(VariableDeclaratorSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            BracketedArgumentListSyntax argumentList = node.ArgumentList;
            if (argumentList != null)
            {
                VisitBracketedArgumentList(argumentList);
            }

            if (!ShouldVisit)
            {
                return;
            }

            EqualsValueClauseSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitEqualsValueClause(initializer);
            }
        }

        public override void VisitVarPattern(VarPatternSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            VariableDesignationSyntax designation = node.Designation;
            if (designation != null)
            {
                VisitVariableDesignation(designation);
            }
        }

        public override void VisitWarningDirectiveTrivia(WarningDirectiveTriviaSyntax node)
        {
        }

        public override void VisitWhenClause(WhenClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }
        }

        public override void VisitWhereClause(WhereClauseSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }
        }

        public override void VisitWhileStatement(WhileStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax condition = node.Condition;
            if (condition != null)
            {
                VisitExpression(condition);
            }

            if (!ShouldVisit)
            {
                return;
            }

            StatementSyntax statement = node.Statement;
            if (statement != null)
            {
                VisitStatement(statement);
            }
        }

        public override void VisitWithExpression(WithExpressionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }

            if (!ShouldVisit)
            {
                return;
            }

            InitializerExpressionSyntax initializer = node.Initializer;
            if (initializer != null)
            {
                VisitInitializerExpression(initializer);
            }
        }

        public override void VisitXmlCDataSection(XmlCDataSectionSyntax node)
        {
        }

        public override void VisitXmlComment(XmlCommentSyntax node)
        {
        }

        public override void VisitXmlCrefAttribute(XmlCrefAttributeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlNameSyntax name = node.Name;
            if (name != null)
            {
                VisitXmlName(name);
            }

            if (!ShouldVisit)
            {
                return;
            }

            CrefSyntax cref = node.Cref;
            if (cref != null)
            {
                VisitCref(cref);
            }
        }

        public override void VisitXmlElement(XmlElementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlElementStartTagSyntax startTag = node.StartTag;
            if (startTag != null)
            {
                VisitXmlElementStartTag(startTag);
            }

            foreach (XmlNodeSyntax xmlNode in node.Content)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitXmlNode(xmlNode);
            }

            if (!ShouldVisit)
            {
                return;
            }

            XmlElementEndTagSyntax endTag = node.EndTag;
            if (endTag != null)
            {
                VisitXmlElementEndTag(endTag);
            }
        }

        public override void VisitXmlElementEndTag(XmlElementEndTagSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlNameSyntax name = node.Name;
            if (name != null)
            {
                VisitXmlName(name);
            }
        }

        public override void VisitXmlElementStartTag(XmlElementStartTagSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlNameSyntax name = node.Name;
            if (name != null)
            {
                VisitXmlName(name);
            }

            foreach (XmlAttributeSyntax xmlAttribute in node.Attributes)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitXmlAttribute(xmlAttribute);
            }
        }

        public override void VisitXmlEmptyElement(XmlEmptyElementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlNameSyntax name = node.Name;
            if (name != null)
            {
                VisitXmlName(name);
            }

            foreach (XmlAttributeSyntax xmlAttribute in node.Attributes)
            {
                if (!ShouldVisit)
                {
                    return;
                }

                VisitXmlAttribute(xmlAttribute);
            }
        }

        public override void VisitXmlName(XmlNameSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlPrefixSyntax prefix = node.Prefix;
            if (prefix != null)
            {
                VisitXmlPrefix(prefix);
            }
        }

        public override void VisitXmlNameAttribute(XmlNameAttributeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlNameSyntax name = node.Name;
            if (name != null)
            {
                VisitXmlName(name);
            }

            if (!ShouldVisit)
            {
                return;
            }

            IdentifierNameSyntax identifier = node.Identifier;
            if (identifier != null)
            {
                VisitIdentifierName(identifier);
            }
        }

        public override void VisitXmlPrefix(XmlPrefixSyntax node)
        {
        }

        public override void VisitXmlProcessingInstruction(XmlProcessingInstructionSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlNameSyntax name = node.Name;
            if (name != null)
            {
                VisitXmlName(name);
            }
        }

        public override void VisitXmlText(XmlTextSyntax node)
        {
        }

        public override void VisitXmlTextAttribute(XmlTextAttributeSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            XmlNameSyntax name = node.Name;
            if (name != null)
            {
                VisitXmlName(name);
            }
        }

        public override void VisitYieldStatement(YieldStatementSyntax node)
        {
            if (!ShouldVisit)
            {
                return;
            }

            ExpressionSyntax expression = node.Expression;
            if (expression != null)
            {
                VisitExpression(expression);
            }
        }

        protected virtual void VisitBaseExpressionColon(BaseExpressionColonSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.ExpressionColon:
                    VisitExpressionColon((ExpressionColonSyntax)node);
                    break;
                case SyntaxKind.NameColon:
                    VisitNameColon((NameColonSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitBaseType(BaseTypeSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.PrimaryConstructorBaseType:
                    VisitPrimaryConstructorBaseType((PrimaryConstructorBaseTypeSyntax)node);
                    break;
                case SyntaxKind.SimpleBaseType:
                    VisitSimpleBaseType((SimpleBaseTypeSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitCref(CrefSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.ConversionOperatorMemberCref:
                    VisitConversionOperatorMemberCref((ConversionOperatorMemberCrefSyntax)node);
                    break;
                case SyntaxKind.IndexerMemberCref:
                    VisitIndexerMemberCref((IndexerMemberCrefSyntax)node);
                    break;
                case SyntaxKind.NameMemberCref:
                    VisitNameMemberCref((NameMemberCrefSyntax)node);
                    break;
                case SyntaxKind.OperatorMemberCref:
                    VisitOperatorMemberCref((OperatorMemberCrefSyntax)node);
                    break;
                case SyntaxKind.QualifiedCref:
                    VisitQualifiedCref((QualifiedCrefSyntax)node);
                    break;
                case SyntaxKind.TypeCref:
                    VisitTypeCref((TypeCrefSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitExpression(ExpressionSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.AliasQualifiedName:
                    VisitAliasQualifiedName((AliasQualifiedNameSyntax)node);
                    break;
                case SyntaxKind.AnonymousMethodExpression:
                    VisitAnonymousMethodExpression((AnonymousMethodExpressionSyntax)node);
                    break;
                case SyntaxKind.AnonymousObjectCreationExpression:
                    VisitAnonymousObjectCreationExpression((AnonymousObjectCreationExpressionSyntax)node);
                    break;
                case SyntaxKind.ArrayCreationExpression:
                    VisitArrayCreationExpression((ArrayCreationExpressionSyntax)node);
                    break;
                case SyntaxKind.ArrayType:
                    VisitArrayType((ArrayTypeSyntax)node);
                    break;
                case SyntaxKind.SimpleAssignmentExpression:
                case SyntaxKind.AddAssignmentExpression:
                case SyntaxKind.SubtractAssignmentExpression:
                case SyntaxKind.MultiplyAssignmentExpression:
                case SyntaxKind.DivideAssignmentExpression:
                case SyntaxKind.ModuloAssignmentExpression:
                case SyntaxKind.AndAssignmentExpression:
                case SyntaxKind.ExclusiveOrAssignmentExpression:
                case SyntaxKind.OrAssignmentExpression:
                case SyntaxKind.LeftShiftAssignmentExpression:
                case SyntaxKind.RightShiftAssignmentExpression:
                case SyntaxKind.CoalesceAssignmentExpression:
                    VisitAssignmentExpression((AssignmentExpressionSyntax)node);
                    break;
                case SyntaxKind.AwaitExpression:
                    VisitAwaitExpression((AwaitExpressionSyntax)node);
                    break;
                case SyntaxKind.BaseExpression:
                    VisitBaseExpression((BaseExpressionSyntax)node);
                    break;
                case SyntaxKind.AddExpression:
                case SyntaxKind.SubtractExpression:
                case SyntaxKind.MultiplyExpression:
                case SyntaxKind.DivideExpression:
                case SyntaxKind.ModuloExpression:
                case SyntaxKind.LeftShiftExpression:
                case SyntaxKind.RightShiftExpression:
                case SyntaxKind.LogicalOrExpression:
                case SyntaxKind.LogicalAndExpression:
                case SyntaxKind.BitwiseOrExpression:
                case SyntaxKind.BitwiseAndExpression:
                case SyntaxKind.ExclusiveOrExpression:
                case SyntaxKind.EqualsExpression:
                case SyntaxKind.NotEqualsExpression:
                case SyntaxKind.LessThanExpression:
                case SyntaxKind.LessThanOrEqualExpression:
                case SyntaxKind.GreaterThanExpression:
                case SyntaxKind.GreaterThanOrEqualExpression:
                case SyntaxKind.IsExpression:
                case SyntaxKind.AsExpression:
                case SyntaxKind.CoalesceExpression:
                    VisitBinaryExpression((BinaryExpressionSyntax)node);
                    break;
                case SyntaxKind.CastExpression:
                    VisitCastExpression((CastExpressionSyntax)node);
                    break;
                case SyntaxKind.ConditionalAccessExpression:
                    VisitConditionalAccessExpression((ConditionalAccessExpressionSyntax)node);
                    break;
                case SyntaxKind.ConditionalExpression:
                    VisitConditionalExpression((ConditionalExpressionSyntax)node);
                    break;
                case SyntaxKind.DeclarationExpression:
                    VisitDeclarationExpression((DeclarationExpressionSyntax)node);
                    break;
                case SyntaxKind.DefaultExpression:
                    VisitDefaultExpression((DefaultExpressionSyntax)node);
                    break;
                case SyntaxKind.ElementAccessExpression:
                    VisitElementAccessExpression((ElementAccessExpressionSyntax)node);
                    break;
                case SyntaxKind.ElementBindingExpression:
                    VisitElementBindingExpression((ElementBindingExpressionSyntax)node);
                    break;
                case SyntaxKind.FunctionPointerType:
                    VisitFunctionPointerType((FunctionPointerTypeSyntax)node);
                    break;
                case SyntaxKind.GenericName:
                    VisitGenericName((GenericNameSyntax)node);
                    break;
                case SyntaxKind.CheckedExpression:
                case SyntaxKind.UncheckedExpression:
                    VisitCheckedExpression((CheckedExpressionSyntax)node);
                    break;
                case SyntaxKind.IdentifierName:
                    VisitIdentifierName((IdentifierNameSyntax)node);
                    break;
                case SyntaxKind.ImplicitArrayCreationExpression:
                    VisitImplicitArrayCreationExpression((ImplicitArrayCreationExpressionSyntax)node);
                    break;
                case SyntaxKind.ImplicitElementAccess:
                    VisitImplicitElementAccess((ImplicitElementAccessSyntax)node);
                    break;
                case SyntaxKind.ImplicitObjectCreationExpression:
                    VisitImplicitObjectCreationExpression((ImplicitObjectCreationExpressionSyntax)node);
                    break;
                case SyntaxKind.ImplicitStackAllocArrayCreationExpression:
                    VisitImplicitStackAllocArrayCreationExpression((ImplicitStackAllocArrayCreationExpressionSyntax)node);
                    break;
                case SyntaxKind.ArrayInitializerExpression:
                case SyntaxKind.CollectionInitializerExpression:
                case SyntaxKind.ComplexElementInitializerExpression:
                case SyntaxKind.ObjectInitializerExpression:
                case SyntaxKind.WithInitializerExpression:
                    VisitInitializerExpression((InitializerExpressionSyntax)node);
                    break;
                case SyntaxKind.InterpolatedStringExpression:
                    VisitInterpolatedStringExpression((InterpolatedStringExpressionSyntax)node);
                    break;
                case SyntaxKind.InvocationExpression:
                    VisitInvocationExpression((InvocationExpressionSyntax)node);
                    break;
                case SyntaxKind.IsPatternExpression:
                    VisitIsPatternExpression((IsPatternExpressionSyntax)node);
                    break;
                case SyntaxKind.ArgListExpression:
                case SyntaxKind.NumericLiteralExpression:
                case SyntaxKind.StringLiteralExpression:
                case SyntaxKind.CharacterLiteralExpression:
                case SyntaxKind.TrueLiteralExpression:
                case SyntaxKind.FalseLiteralExpression:
                case SyntaxKind.NullLiteralExpression:
                case SyntaxKind.DefaultLiteralExpression:
                    VisitLiteralExpression((LiteralExpressionSyntax)node);
                    break;
                case SyntaxKind.MakeRefExpression:
                    VisitMakeRefExpression((MakeRefExpressionSyntax)node);
                    break;
                case SyntaxKind.PointerMemberAccessExpression:
                case SyntaxKind.SimpleMemberAccessExpression:
                    VisitMemberAccessExpression((MemberAccessExpressionSyntax)node);
                    break;
                case SyntaxKind.MemberBindingExpression:
                    VisitMemberBindingExpression((MemberBindingExpressionSyntax)node);
                    break;
                case SyntaxKind.NullableType:
                    VisitNullableType((NullableTypeSyntax)node);
                    break;
                case SyntaxKind.ObjectCreationExpression:
                    VisitObjectCreationExpression((ObjectCreationExpressionSyntax)node);
                    break;
                case SyntaxKind.OmittedArraySizeExpression:
                    VisitOmittedArraySizeExpression((OmittedArraySizeExpressionSyntax)node);
                    break;
                case SyntaxKind.OmittedTypeArgument:
                    VisitOmittedTypeArgument((OmittedTypeArgumentSyntax)node);
                    break;
                case SyntaxKind.ParenthesizedExpression:
                    VisitParenthesizedExpression((ParenthesizedExpressionSyntax)node);
                    break;
                case SyntaxKind.ParenthesizedLambdaExpression:
                    VisitParenthesizedLambdaExpression((ParenthesizedLambdaExpressionSyntax)node);
                    break;
                case SyntaxKind.PointerType:
                    VisitPointerType((PointerTypeSyntax)node);
                    break;
                case SyntaxKind.PostDecrementExpression:
                case SyntaxKind.PostIncrementExpression:
                case SyntaxKind.SuppressNullableWarningExpression:
                    VisitPostfixUnaryExpression((PostfixUnaryExpressionSyntax)node);
                    break;
                case SyntaxKind.PredefinedType:
                    VisitPredefinedType((PredefinedTypeSyntax)node);
                    break;
                case SyntaxKind.UnaryPlusExpression:
                case SyntaxKind.UnaryMinusExpression:
                case SyntaxKind.BitwiseNotExpression:
                case SyntaxKind.LogicalNotExpression:
                case SyntaxKind.PreIncrementExpression:
                case SyntaxKind.PreDecrementExpression:
                case SyntaxKind.AddressOfExpression:
                case SyntaxKind.PointerIndirectionExpression:
                case SyntaxKind.IndexExpression:
                    VisitPrefixUnaryExpression((PrefixUnaryExpressionSyntax)node);
                    break;
                case SyntaxKind.QualifiedName:
                    VisitQualifiedName((QualifiedNameSyntax)node);
                    break;
                case SyntaxKind.QueryExpression:
                    VisitQueryExpression((QueryExpressionSyntax)node);
                    break;
                case SyntaxKind.RangeExpression:
                    VisitRangeExpression((RangeExpressionSyntax)node);
                    break;
                case SyntaxKind.RefExpression:
                    VisitRefExpression((RefExpressionSyntax)node);
                    break;
                case SyntaxKind.RefTypeExpression:
                    VisitRefTypeExpression((RefTypeExpressionSyntax)node);
                    break;
                case SyntaxKind.RefType:
                    VisitRefType((RefTypeSyntax)node);
                    break;
                case SyntaxKind.RefValueExpression:
                    VisitRefValueExpression((RefValueExpressionSyntax)node);
                    break;
                case SyntaxKind.SimpleLambdaExpression:
                    VisitSimpleLambdaExpression((SimpleLambdaExpressionSyntax)node);
                    break;
                case SyntaxKind.SizeOfExpression:
                    VisitSizeOfExpression((SizeOfExpressionSyntax)node);
                    break;
                case SyntaxKind.StackAllocArrayCreationExpression:
                    VisitStackAllocArrayCreationExpression((StackAllocArrayCreationExpressionSyntax)node);
                    break;
                case SyntaxKind.SwitchExpression:
                    VisitSwitchExpression((SwitchExpressionSyntax)node);
                    break;
                case SyntaxKind.ThisExpression:
                    VisitThisExpression((ThisExpressionSyntax)node);
                    break;
                case SyntaxKind.ThrowExpression:
                    VisitThrowExpression((ThrowExpressionSyntax)node);
                    break;
                case SyntaxKind.TupleExpression:
                    VisitTupleExpression((TupleExpressionSyntax)node);
                    break;
                case SyntaxKind.TupleType:
                    VisitTupleType((TupleTypeSyntax)node);
                    break;
                case SyntaxKind.TypeOfExpression:
                    VisitTypeOfExpression((TypeOfExpressionSyntax)node);
                    break;
                case SyntaxKind.WithExpression:
                    VisitWithExpression((WithExpressionSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitInterpolatedStringContent(InterpolatedStringContentSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.InterpolatedStringText:
                    VisitInterpolatedStringText((InterpolatedStringTextSyntax)node);
                    break;
                case SyntaxKind.Interpolation:
                    VisitInterpolation((InterpolationSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitMemberCref(MemberCrefSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.ConversionOperatorMemberCref:
                    VisitConversionOperatorMemberCref((ConversionOperatorMemberCrefSyntax)node);
                    break;
                case SyntaxKind.IndexerMemberCref:
                    VisitIndexerMemberCref((IndexerMemberCrefSyntax)node);
                    break;
                case SyntaxKind.NameMemberCref:
                    VisitNameMemberCref((NameMemberCrefSyntax)node);
                    break;
                case SyntaxKind.OperatorMemberCref:
                    VisitOperatorMemberCref((OperatorMemberCrefSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitMemberDeclaration(MemberDeclarationSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.ClassDeclaration:
                    VisitClassDeclaration((ClassDeclarationSyntax)node);
                    break;
                case SyntaxKind.ConstructorDeclaration:
                    VisitConstructorDeclaration((ConstructorDeclarationSyntax)node);
                    break;
                case SyntaxKind.ConversionOperatorDeclaration:
                    VisitConversionOperatorDeclaration((ConversionOperatorDeclarationSyntax)node);
                    break;
                case SyntaxKind.DelegateDeclaration:
                    VisitDelegateDeclaration((DelegateDeclarationSyntax)node);
                    break;
                case SyntaxKind.DestructorDeclaration:
                    VisitDestructorDeclaration((DestructorDeclarationSyntax)node);
                    break;
                case SyntaxKind.EnumDeclaration:
                    VisitEnumDeclaration((EnumDeclarationSyntax)node);
                    break;
                case SyntaxKind.EnumMemberDeclaration:
                    VisitEnumMemberDeclaration((EnumMemberDeclarationSyntax)node);
                    break;
                case SyntaxKind.EventDeclaration:
                    VisitEventDeclaration((EventDeclarationSyntax)node);
                    break;
                case SyntaxKind.EventFieldDeclaration:
                    VisitEventFieldDeclaration((EventFieldDeclarationSyntax)node);
                    break;
                case SyntaxKind.FieldDeclaration:
                    VisitFieldDeclaration((FieldDeclarationSyntax)node);
                    break;
                case SyntaxKind.FileScopedNamespaceDeclaration:
                    VisitFileScopedNamespaceDeclaration((FileScopedNamespaceDeclarationSyntax)node);
                    break;
                case SyntaxKind.GlobalStatement:
                    VisitGlobalStatement((GlobalStatementSyntax)node);
                    break;
                case SyntaxKind.IncompleteMember:
                    VisitIncompleteMember((IncompleteMemberSyntax)node);
                    break;
                case SyntaxKind.IndexerDeclaration:
                    VisitIndexerDeclaration((IndexerDeclarationSyntax)node);
                    break;
                case SyntaxKind.InterfaceDeclaration:
                    VisitInterfaceDeclaration((InterfaceDeclarationSyntax)node);
                    break;
                case SyntaxKind.MethodDeclaration:
                    VisitMethodDeclaration((MethodDeclarationSyntax)node);
                    break;
                case SyntaxKind.NamespaceDeclaration:
                    VisitNamespaceDeclaration((NamespaceDeclarationSyntax)node);
                    break;
                case SyntaxKind.OperatorDeclaration:
                    VisitOperatorDeclaration((OperatorDeclarationSyntax)node);
                    break;
                case SyntaxKind.PropertyDeclaration:
                    VisitPropertyDeclaration((PropertyDeclarationSyntax)node);
                    break;
                case SyntaxKind.RecordDeclaration:
                case SyntaxKind.RecordStructDeclaration:
                    VisitRecordDeclaration((RecordDeclarationSyntax)node);
                    break;
                case SyntaxKind.StructDeclaration:
                    VisitStructDeclaration((StructDeclarationSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitPattern(PatternSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.AndPattern:
                case SyntaxKind.OrPattern:
                    VisitBinaryPattern((BinaryPatternSyntax)node);
                    break;
                case SyntaxKind.ConstantPattern:
                    VisitConstantPattern((ConstantPatternSyntax)node);
                    break;
                case SyntaxKind.DeclarationPattern:
                    VisitDeclarationPattern((DeclarationPatternSyntax)node);
                    break;
                case SyntaxKind.DiscardPattern:
                    VisitDiscardPattern((DiscardPatternSyntax)node);
                    break;
                case SyntaxKind.ParenthesizedPattern:
                    VisitParenthesizedPattern((ParenthesizedPatternSyntax)node);
                    break;
                case SyntaxKind.RecursivePattern:
                    VisitRecursivePattern((RecursivePatternSyntax)node);
                    break;
                case SyntaxKind.RelationalPattern:
                    VisitRelationalPattern((RelationalPatternSyntax)node);
                    break;
                case SyntaxKind.TypePattern:
                    VisitTypePattern((TypePatternSyntax)node);
                    break;
                case SyntaxKind.NotPattern:
                    VisitUnaryPattern((UnaryPatternSyntax)node);
                    break;
                case SyntaxKind.VarPattern:
                    VisitVarPattern((VarPatternSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitQueryClause(QueryClauseSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.FromClause:
                    VisitFromClause((FromClauseSyntax)node);
                    break;
                case SyntaxKind.JoinClause:
                    VisitJoinClause((JoinClauseSyntax)node);
                    break;
                case SyntaxKind.LetClause:
                    VisitLetClause((LetClauseSyntax)node);
                    break;
                case SyntaxKind.OrderByClause:
                    VisitOrderByClause((OrderByClauseSyntax)node);
                    break;
                case SyntaxKind.WhereClause:
                    VisitWhereClause((WhereClauseSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitSelectOrGroupClause(SelectOrGroupClauseSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.GroupClause:
                    VisitGroupClause((GroupClauseSyntax)node);
                    break;
                case SyntaxKind.SelectClause:
                    VisitSelectClause((SelectClauseSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitSimpleName(SimpleNameSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.GenericName:
                    VisitGenericName((GenericNameSyntax)node);
                    break;
                case SyntaxKind.IdentifierName:
                    VisitIdentifierName((IdentifierNameSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitStatement(StatementSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.Block:
                    VisitBlock((BlockSyntax)node);
                    break;
                case SyntaxKind.BreakStatement:
                    VisitBreakStatement((BreakStatementSyntax)node);
                    break;
                case SyntaxKind.ContinueStatement:
                    VisitContinueStatement((ContinueStatementSyntax)node);
                    break;
                case SyntaxKind.DoStatement:
                    VisitDoStatement((DoStatementSyntax)node);
                    break;
                case SyntaxKind.EmptyStatement:
                    VisitEmptyStatement((EmptyStatementSyntax)node);
                    break;
                case SyntaxKind.ExpressionStatement:
                    VisitExpressionStatement((ExpressionStatementSyntax)node);
                    break;
                case SyntaxKind.FixedStatement:
                    VisitFixedStatement((FixedStatementSyntax)node);
                    break;
                case SyntaxKind.ForEachStatement:
                    VisitForEachStatement((ForEachStatementSyntax)node);
                    break;
                case SyntaxKind.ForEachVariableStatement:
                    VisitForEachVariableStatement((ForEachVariableStatementSyntax)node);
                    break;
                case SyntaxKind.ForStatement:
                    VisitForStatement((ForStatementSyntax)node);
                    break;
                case SyntaxKind.GotoStatement:
                case SyntaxKind.GotoCaseStatement:
                case SyntaxKind.GotoDefaultStatement:
                    VisitGotoStatement((GotoStatementSyntax)node);
                    break;
                case SyntaxKind.CheckedStatement:
                case SyntaxKind.UncheckedStatement:
                    VisitCheckedStatement((CheckedStatementSyntax)node);
                    break;
                case SyntaxKind.IfStatement:
                    VisitIfStatement((IfStatementSyntax)node);
                    break;
                case SyntaxKind.LabeledStatement:
                    VisitLabeledStatement((LabeledStatementSyntax)node);
                    break;
                case SyntaxKind.LocalDeclarationStatement:
                    VisitLocalDeclarationStatement((LocalDeclarationStatementSyntax)node);
                    break;
                case SyntaxKind.LocalFunctionStatement:
                    VisitLocalFunctionStatement((LocalFunctionStatementSyntax)node);
                    break;
                case SyntaxKind.LockStatement:
                    VisitLockStatement((LockStatementSyntax)node);
                    break;
                case SyntaxKind.ReturnStatement:
                    VisitReturnStatement((ReturnStatementSyntax)node);
                    break;
                case SyntaxKind.SwitchStatement:
                    VisitSwitchStatement((SwitchStatementSyntax)node);
                    break;
                case SyntaxKind.ThrowStatement:
                    VisitThrowStatement((ThrowStatementSyntax)node);
                    break;
                case SyntaxKind.TryStatement:
                    VisitTryStatement((TryStatementSyntax)node);
                    break;
                case SyntaxKind.UnsafeStatement:
                    VisitUnsafeStatement((UnsafeStatementSyntax)node);
                    break;
                case SyntaxKind.UsingStatement:
                    VisitUsingStatement((UsingStatementSyntax)node);
                    break;
                case SyntaxKind.WhileStatement:
                    VisitWhileStatement((WhileStatementSyntax)node);
                    break;
                case SyntaxKind.YieldBreakStatement:
                case SyntaxKind.YieldReturnStatement:
                    VisitYieldStatement((YieldStatementSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitSwitchLabel(SwitchLabelSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.CasePatternSwitchLabel:
                    VisitCasePatternSwitchLabel((CasePatternSwitchLabelSyntax)node);
                    break;
                case SyntaxKind.CaseSwitchLabel:
                    VisitCaseSwitchLabel((CaseSwitchLabelSyntax)node);
                    break;
                case SyntaxKind.DefaultSwitchLabel:
                    VisitDefaultSwitchLabel((DefaultSwitchLabelSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitTypeParameterConstraint(TypeParameterConstraintSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.ClassConstraint:
                case SyntaxKind.StructConstraint:
                    VisitClassOrStructConstraint((ClassOrStructConstraintSyntax)node);
                    break;
                case SyntaxKind.ConstructorConstraint:
                    VisitConstructorConstraint((ConstructorConstraintSyntax)node);
                    break;
                case SyntaxKind.DefaultConstraint:
                    VisitDefaultConstraint((DefaultConstraintSyntax)node);
                    break;
                case SyntaxKind.TypeConstraint:
                    VisitTypeConstraint((TypeConstraintSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitType(TypeSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.AliasQualifiedName:
                    VisitAliasQualifiedName((AliasQualifiedNameSyntax)node);
                    break;
                case SyntaxKind.ArrayType:
                    VisitArrayType((ArrayTypeSyntax)node);
                    break;
                case SyntaxKind.FunctionPointerType:
                    VisitFunctionPointerType((FunctionPointerTypeSyntax)node);
                    break;
                case SyntaxKind.GenericName:
                    VisitGenericName((GenericNameSyntax)node);
                    break;
                case SyntaxKind.IdentifierName:
                    VisitIdentifierName((IdentifierNameSyntax)node);
                    break;
                case SyntaxKind.NullableType:
                    VisitNullableType((NullableTypeSyntax)node);
                    break;
                case SyntaxKind.OmittedTypeArgument:
                    VisitOmittedTypeArgument((OmittedTypeArgumentSyntax)node);
                    break;
                case SyntaxKind.PointerType:
                    VisitPointerType((PointerTypeSyntax)node);
                    break;
                case SyntaxKind.PredefinedType:
                    VisitPredefinedType((PredefinedTypeSyntax)node);
                    break;
                case SyntaxKind.QualifiedName:
                    VisitQualifiedName((QualifiedNameSyntax)node);
                    break;
                case SyntaxKind.RefType:
                    VisitRefType((RefTypeSyntax)node);
                    break;
                case SyntaxKind.TupleType:
                    VisitTupleType((TupleTypeSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitVariableDesignation(VariableDesignationSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.DiscardDesignation:
                    VisitDiscardDesignation((DiscardDesignationSyntax)node);
                    break;
                case SyntaxKind.ParenthesizedVariableDesignation:
                    VisitParenthesizedVariableDesignation((ParenthesizedVariableDesignationSyntax)node);
                    break;
                case SyntaxKind.SingleVariableDesignation:
                    VisitSingleVariableDesignation((SingleVariableDesignationSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitXmlAttribute(XmlAttributeSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.XmlCrefAttribute:
                    VisitXmlCrefAttribute((XmlCrefAttributeSyntax)node);
                    break;
                case SyntaxKind.XmlNameAttribute:
                    VisitXmlNameAttribute((XmlNameAttributeSyntax)node);
                    break;
                case SyntaxKind.XmlTextAttribute:
                    VisitXmlTextAttribute((XmlTextAttributeSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }

        protected virtual void VisitXmlNode(XmlNodeSyntax node)
        {
            switch (node.Kind())
            {
                case SyntaxKind.XmlCDataSection:
                    VisitXmlCDataSection((XmlCDataSectionSyntax)node);
                    break;
                case SyntaxKind.XmlComment:
                    VisitXmlComment((XmlCommentSyntax)node);
                    break;
                case SyntaxKind.XmlElement:
                    VisitXmlElement((XmlElementSyntax)node);
                    break;
                case SyntaxKind.XmlEmptyElement:
                    VisitXmlEmptyElement((XmlEmptyElementSyntax)node);
                    break;
                case SyntaxKind.XmlProcessingInstruction:
                    VisitXmlProcessingInstruction((XmlProcessingInstructionSyntax)node);
                    break;
                case SyntaxKind.XmlText:
                    VisitXmlText((XmlTextSyntax)node);
                    break;
                default:
                    Debug.Fail($"Unrecognized kind '{node.Kind()}'.");
                    base.Visit(node);
                    break;
            }
        }
    }
}